/*
 *  UCIAgentBridgeClass.h
 *  UCIAgent
 *
 *  Created by Christian Weykopf on 11.12.09.
 *  Copyright 2009 STARFACE GmbH. All rights reserved.
 *
 */

#import <Cocoa/Cocoa.h>

/*!
    @class      UCIAgentBridge
    @abstract   The interface class for the UCIAgent.
    @discussion This class is the interface with UCIAgent.
 
                It provides a way to detect if the Agent is running and launch it if not.
                Several methods to comminicate with the UCIAgent are provied.
 */
@interface UCIAgentBridge : NSObject
{
}

/*!
    @method     isUCIAgentRunning
    @abstract   Detects whether the UCIAgent is installed.
    @discussion Detemines if the UCIAgent is installed.
    @result     YES if UCIAgent is installed, NO otherwise.
 */
+ (BOOL) isUCIAgentInstalled;

/*!
    @method     isUCIAgentRunning
    @abstract   Detects whether UCIAgent is running.
    @discussion Searches through the process list to find whether UCIAgent is running and returns the answer.
    @result     YES if UCIAgent is running, NO otherwise.
 */
+ (BOOL) isUCIAgentRunning;

/*!
    @method     isUCIAgentRunningForName
    @abstract   Detects whether UCIAgent is running.
    @discussion Searches through the process list to find whether UCIAgent is running for a specific user and returns the answer.
    @param      userName    The user in question.
    @result     YES if UCIAgent is running, NO otherwise.
 */
+ (BOOL) isUCIAgentRunningForName: (NSString *)userName;

#pragma mark -

/*!
    @method     setDelegate:
    @abstract   Set the object which will be responsible for receiving UCIAgentBridge information.
    @discussion This must be called before using UCIAgentBridge.
 
                The methods in the UCIAgentBridgeDelegate_InformalProtocol are individually optional.
                They provide a greater degree of interaction between the application and UCIAgent such as informing
                the application when one of its Growl notifications is clicked by the user.
 
                When this method is called, UCIAgent will be launched if it is not already running, and the application will be
                connected to the Agent.
 
    @param      inDelegate The delegate for the UCIAgentBridge. It should conform to the UCIAgentBridgeDelegate_InformalProtocol protocol.
 */
+ (void) setDelegate:(id)inDelegate;

/*!
    @method     delegate
    @abstract   Return the object responsible receiving UCIAgentBridge information.
    @discussion See setDelegate: for details.
    @result     The delegate.
 */
+ (id) delegate;

#pragma mark -
/*!
    @method     startUCIAgent
    @abstract   Starts the UCIAgent if not already running.
    @discussion Starts UCIAgent. It wait not for the complete startup of UCIAgent.
                The UCIAgent reports a succsessful start later.
    @result     YES if UCIAgent could started, NO otherwise.
 */
+ (BOOL) startUCIAgent;

+ (BOOL) startUCIAgentForName: (NSString *)userName;

/*!
    @method     stopUCIAgent
    @abstract   Stops UCIAgent.
    @discussion UCIAgent is signalled to stop. The method doesn't wait for the shutdown.
                UCIAgent reports a shutdown later.
 */
+ (void) stopUCIAgent;

#pragma mark -
/*!
    @method     connectToUCIAgent
    @abstract   Connects to UCIAgent.
    @discussion Installs a DO connection to UCIAgent if not already conneced and stores the proxy object.
    @result     YES if the connect is established, NO otherwise.
 */
+ (BOOL) connectToUCIAgent;

/*!
    @method     disconnectFromUCIAgent
    @abstract   Closes the connection to UCIAgent.
    @discussion Before disconnecting release all objects obtained from the connection such as ucpClient and ucpServer.
 */
+ (void) disconnectFromUCIAgent;

#pragma mark -
/*!
    @method     ucpServer
    @abstract   Obtains the UCPServer object.
    @discussion Use the ucpserver to communicate with the UCPServer.
                The ucpServer forwards unknown methods to the different services.
 */
+ (id) ucpServer;

/*!
    @method     ucpClient
    @abstract   Obtains the UCPClient object.
    @discussion Use the ucpClient to communicate with the client.
                The ucpClient forwards unknown methods to the different services.
 */
+ (id) ucpClient;

#pragma mark -
/*!
    @method     sendFaxFile:faxId:digestName:
    @abstract   Sends a fax file.
    @discussion Sends a file for faxing. use this message right after successful placing a fax call.
    @param      faxFilePath     The file to send.
    @param      faxId           The faxId obtained from placing the fax call
    @param      digestName      The digest mehod to secure the file transfer. Digestname can either SHA or MD5.
 */
+ (BOOL) sendFaxFile: (NSString *)faxFilePath faxId: (NSString *)faxId digestName: (NSString *)digestName;

/*!
    @method     connectToRPCServer
    @abstract   Sends login to the RPC server.
    @discussion Sends login to the RPC server if one is configured to do.
 */
+ (BOOL) connectToRPCServer;

+ (oneway void) postFaxStatus: (NSNumber *)status jobId: (NSString *)jobId;

+ (void) setDebugLogging: (BOOL)onOff;

@end

#pragma mark -
/*!
    @protocol    NSObject(UCIAgentBridgeDelegate_InformalProtocol)
    @abstract    Methods which may be optionally implemented by the delegate.
    @discussion  The methods in this informal protocol will only be called if implemented by the delegate.
 */
@interface NSObject (UCIAgentBridgeDelegate_InformalProtocol)

/*!
    @method     agentIsReady
    @abstract   Informs the delegate that the UCIAgent has launched.
    @discussion UCIAgent is up and running. The application can take further actions.
                Additionally a notification with name "UCIAgentIsReady" is posted.
 */
- (void) agentIsReady;

/*!
    @method     agentTerminated
    @abstract   Informs the delegte that the UCIAgent has shut down.
    @discussion UCIAgent is no longer running. Don't take any actions.
                Additionally a notification with name "UCIAgentIsTerminated" is posted.
 */
- (void) agentTerminated;

/*!
    @method     agentDied
    @abstract   Informs the delegate that the connection has died.
    @discussion The connection to UCIAgent is no longer working. Don't take any actions.
                Additionally a notification with name "UCIAgentDied" is posted.
 */
- (void) agentDied;

/*!
    @method     userName
    @abstract   Asks the delegate for a user name.
    @discussion The user name is needed to construct the portname for the server.
                Implement if the user name ist different from NSUserName ().
 */
- (NSString *) userName;
@end